
/**
 * -----------------------------------------------------
 * VisualU is a singleton object that educates DOM elements,
 * usually of type HTMLDivElement, to be visuals.
 * A visual will automatically size itself to fill any pane
 * that it is placed into.
 * VisualU can also create a visual from scratch.
 */
VisualU = {
  /**
   * VisualU.teach(element) teaches element to be a visual.
   */
  teach: function(element) {
    var style = element.style;
    style.position = "absolute";
    style.left = "0";
    style.right = "0";
    style.top = "0";
    style.bottom = "0";
  },
  /**
   * VisualU.create() returns a new HTMLDivElement visual.
   */
  create: function() {
    var visual = document.createElement("div");
    VisualU.teach(visual);
    return visual;
  }
}

/**
 * -----------------------------------------------------
 * PaneU is a singleton object that educates DOM elements,
 * usually of type HTMLDivElement, to be panes.
 * A pane has no appearence on its own, but can contain a
 * visual.
 * PaneU can also create a pane from scratch.
 */
PaneU = {
  /**
   * PaneU.teach(element, name) teaches element to be a
   * pane.
   * The parameter name is used to identify element in
   * its toString() method.
   */
  teach: function(element, name) {
    element.IDENTIFIER = name;
    element.toString = this.toString;
    var style = element.style;
    style.position = "absolute";
    style.padding = "0px";
    style.margin = "0px";
    style.borderStyle = "none";
  },
  /**
   * pane.toString() returns a string identifying pane.
   * The string has the form "Pane<name>", where name
   * is the identifier given to pane.
   */
  toString : function() {
    return "Pane<" + this.IDENTIFIER + ">";
  },
  /**
   * PaneU.create(name) returns a new HTMLDivElement pane
   * whose identifier is name.
   */
  create: function(name) {
    var pane = document.createElement("div");
    PaneU.teach(pane, name);
    return pane;
  }
}

/**
 * -----------------------------------------------------
 * Layout is a singleton object that lays out visuals in
 * a browser window.
 * After Layout.initialize() is invoked, Layout always has
 * a current pane.
 * The current pane can be accessed as Layout.currentPane.
 * It will only be null if all of the browser window space
 * has been allocated.
 *
 * A visual can be added to the current pane by invoking
 * one of the methods putLeft(), putRight(), putTop(),
 * putBottom(), or putRest() with the visual as the first
 * parameter.
 * The first four of these methods have CSS length
 * parameters to specify one of the dimension of the added
 * visual.
 * These methods will set Layout.currentPane to be a new
 * pane that occupies the unused window space.
 * The putRest() method gives all of the current pane space
 * to its visual parameter and sets Layout.currentPane to
 * null.
 *
 * Client code can retain a copy of Layout.currentPane for
 * future use.
 * A new visual can be placed in the pane by invoking
 * setVisual() with the pane as the first parameter and the
 * new visual as the second parameter.
 *
 * Invokations of the setVisual() method can be logged by
 * setting Layout.debug.SET_VISUAL to true.
 * Since all of the "put" methods invoke setVisual, their
 * invokations will also be logged.
 *
 * Changes to Layout.currentPane can be logged by setting
 * Layout.debug.CURRENT_PANE to true.
 */
Layout = {
  debug: {},
  /**
   * intent: private
   * Layout.setCurrentPane(pane) sets Layout.currentPane
   * to pane.
   * If Layout.debug.CURRENT_PANE is true this change
   * will be logged.
   */
  setCurrentPane: function(pane) {
    Layout.currentPane = pane;
    if (Layout.debug.CURRENT_PANE) {
      var message = "Layout.currentPane is now ";
      if (Layout.currentPane == null) {
        message += "null"
      } else {
        message += Layout.currentPane.toString();
      }
      Log.add(message);
    }
  },
  /**
   * Layout.initialize() configures document.body to be
   * the root pane and sets Layout.currentPane to be
   * document.body.
   *
   */
  initialize: function() {
    var rootPane = document.body;
    VisualU.teach(rootPane);
    PaneU.teach(rootPane, "root");
    rootPane.style.overflow = "hidden";
    Layout.setCurrentPane(rootPane);
  },
  /**
   * Layout.putLeft(visual, width) sets the width of visual
   * to width and adds it into the left side of the current
   * pane.
   * Then the current pane is replaced by a new pane that
   * occupies the leftover window space.
   */
  putLeft: function(visual, width) {
    var currentPane = Layout.currentPane;
    var leftPane = PaneU.create(currentPane.IDENTIFIER + ".left");
    Layout.setVisual(leftPane, visual);
    leftPane.style.width = width;
    leftPane.style.left = "0px";
    leftPane.style.top = "0px";
    leftPane.style.bottom = "0px";
    Layout.currentPane.appendChild(leftPane);
    var rightPane = PaneU.create(currentPane.IDENTIFIER + ".right");
    rightPane.style.left = width;
    rightPane.style.right = "0px";
    rightPane.style.top = "0px";
    rightPane.style.bottom = "0px";
    Layout.currentPane.appendChild(rightPane);
    Layout.setCurrentPane(rightPane);
  },
  /**
   * Layout.putRight(visual, width) sets the width of visual
   * to width and adds it into the right side of current
   * pane.
   * Then the current pane is replaced by a new pane that
   * occupies the leftover window space.
   */
  putRight: function(visual, width) {
    var currentPane = Layout.currentPane;
    var rightPane = PaneU.create(currentPane.IDENTIFIER + ".right");
    Layout.setVisual(rightPane, visual);
    rightPane.style.width = width;
    rightPane.style.right = "0px";
    rightPane.style.top = "0px";
    rightPane.style.bottom = "0px";
    Layout.currentPane.appendChild(rightPane);
    var leftPane = PaneU.create(currentPane.IDENTIFIER + ".left");
    leftPane.style.right = width;
    leftPane.style.left = "0px";
    leftPane.style.top = "0px";
    leftPane.style.bottom = "0px";
    Layout.currentPane.appendChild(leftPane);
    Layout.setCurrentPane(leftPane);
  },
  /**
   * Layout.putTop(visual, height) sets the height of visual
   * to height and adds it into the top side of the current
   * pane.
   * Then the current pane is replaced by a new pane that
   * occupies the leftover window space.
   */
  putTop: function(visual, height) {
    var currentPane = Layout.currentPane;
    var topPane = PaneU.create(currentPane.IDENTIFIER + ".top");
    Layout.setVisual(topPane, visual);
    topPane.style.height = height;
    topPane.style.left = "0px";
    topPane.style.right = "0px";
    topPane.style.top = "0px";
    Layout.currentPane.appendChild(topPane);
    var bottomPane = PaneU.create(currentPane.IDENTIFIER + ".bottom");
    bottomPane.style.top = height;
    bottomPane.style.left = "0px";
    bottomPane.style.right = "0px";
    bottomPane.style.bottom = "0px";
    Layout.currentPane.appendChild(bottomPane);
    Layout.setCurrentPane(bottomPane);
  },
  /**
   * Layout.putBottom(visual, height) sets the height of visual
   * to height and adds it into the bottom side of the current
   * pane.
   * Then the current pane is replaced by a new pane that
   * occupies the leftover window space.
   */
  putBottom: function(visual, height) {
    var currentPane = Layout.currentPane;
    var bottomPane = PaneU.create(currentPane.IDENTIFIER + ".bottom");
    Layout.setVisual(bottomPane, visual);
    bottomPane.style.height = height;
    bottomPane.style.left = "0px";
    bottomPane.style.right = "0px";
    bottomPane.style.bottom = "0px";
    Layout.currentPane.appendChild(bottomPane);
    var topPane = PaneU.create(currentPane.IDENTIFIER + ".top");
    topPane.style.bottom = height;
    topPane.style.left = "0px";
    topPane.style.right = "0px";
    topPane.style.top = "0px";
    Layout.currentPane.appendChild(topPane);
    Layout.setCurrentPane(topPane);
  },
  /**
   * Layout.putRest(visual) makes visual occupy the entire
   * current pane.
   * Then the current pane is replaced by null.
   */
  putRest: function(visual) {
    Layout.setVisual(Layout.currentPane, visual);
    Layout.setCurrentPane(null);
  },
  /**
   * Layout.setVisual(pane, visual) replaces the old visual
   * of pane by visual.
   */
  setVisual: function(pane, visual) {
    DOM.appendChild(pane, visual, "Layout.setVisual");
    if (pane.VISUAL) {
      DOM.removeChild(pane, pane.VISUAL, "Layout.setVisual");
    }
    pane.VISUAL = visual;
    if (Layout.debug.SET_VISUAL) {
      var message = pane.toString();
      message += ".VISUAL changed to ";
      message += pane.VISUAL.toString();
      Log.add(message);
    }
  }
};
